@@ -1,4 +1,3 @@
-# DeviceCodeCredential ----
 #' Device code credential authentication
 #'
 #' @description
@@ -32,7 +31,8 @@
 #' }
 DeviceCodeCredential <- R6::R6Class(
   classname = "DeviceCodeCredential",
-  inherit = InteractiveCredential, ,
+  inherit = InteractiveCredential,
+  ,
   public = list(
     #' @description
     #' Create a new device code credential
@@ -46,13 +46,19 @@ DeviceCodeCredential <- R6::R6Class(
     #'   for disk-based caching or `"memory"` for in-memory caching. Defaults to `"disk"`.
     #' @param offline A logical value indicating whether to request offline access
     #'   (refresh tokens). Defaults to `TRUE`.
+    #' @param interactive A logical value indicating whether this credential
+    #'   requires user interaction. Defaults to `TRUE`.
     #'
     #' @return A new `DeviceCodeCredential` object
-    initialize = function(scope = NULL,
-                          tenant_id = NULL,
-                          client_id = NULL,
-                          use_cache = "disk",
-                          offline = TRUE) {
+    initialize = function(
+      scope = NULL,
+      tenant_id = NULL,
+      client_id = NULL,
+      use_cache = "disk",
+      offline = TRUE,
+      interactive = TRUE
+    ) {
+      self$interactive <- interactive
       super$initialize(
         scope = scope,
         tenant_id = tenant_id,
@@ -71,9 +77,15 @@ DeviceCodeCredential <- R6::R6Class(
     #'
     #' @return An [httr2::oauth_token()] object containing the access token
     get_token = function(reauth = FALSE) {
+      flow_fun <- if (self$is_interactive()) {
+        httr2::oauth_flow_device
+      } else {
+        \(...) cli::cli_abort("non-interactive session")
+      }
+
       httr2::oauth_token_cached(
         client = self$.oauth_client,
-        flow = httr2::oauth_flow_device,
+        flow = flow_fun,
         cache_disk = self$.use_cache == "disk",
         cache_key = self$.cache_key,
         flow_params = list(
@@ -103,7 +115,6 @@ DeviceCodeCredential <- R6::R6Class(
 )
 
 
-# AuthCodeCredential ----
 #' Authorization code credential authentication
 #'
 #' @description
@@ -142,7 +153,8 @@ DeviceCodeCredential <- R6::R6Class(
 #' }
 AuthCodeCredential <- R6::R6Class(
   classname = "AuthCodeCredential",
-  inherit = InteractiveCredential, ,
+  inherit = InteractiveCredential,
+  ,
   public = list(
     #' @description
     #' Create a new authorization code credential
@@ -158,14 +170,21 @@ AuthCodeCredential <- R6::R6Class(
     #'   (refresh tokens). Defaults to `TRUE`.
     #' @param redirect_uri A character string specifying the redirect URI registered
     #'   with the application. Defaults to [default_redirect_uri()].
+    #' @param interactive A logical value indicating whether this credential
+    #'   requires user interaction. Defaults to `TRUE`.
     #'
     #' @return A new `AuthCodeCredential` object
-    initialize = function(scope = NULL,
-                          tenant_id = NULL,
-                          client_id = NULL,
-                          use_cache = "disk",
-                          offline = TRUE,
-                          redirect_uri = default_redirect_uri()) {
+    initialize = function(
+      scope = NULL,
+      tenant_id = NULL,
+      client_id = NULL,
+      use_cache = "disk",
+      offline = TRUE,
+      redirect_uri = default_redirect_uri(),
+      interactive = TRUE
+    ) {
+      self$interactive <- interactive
+
       super$initialize(
         scope = scope,
         tenant_id = tenant_id,
@@ -186,9 +205,15 @@ AuthCodeCredential <- R6::R6Class(
     #'
     #' @return An [httr2::oauth_token()] object containing the access token
     get_token = function(reauth = FALSE) {
+      flow_fun <- if (self$is_interactive()) {
+        httr2::oauth_flow_auth_code
+      } else {
+        \(...) cli::cli_abort("non-interactive session")
+      }
+
       httr2::oauth_token_cached(
         client = self$.oauth_client,
-        flow = httr2::oauth_flow_auth_code,
+        flow = flow_fun,
         cache_disk = self$.use_cache == "disk",
         cache_key = self$.cache_key,
         flow_params = list(
@@ -219,7 +244,7 @@ AuthCodeCredential <- R6::R6Class(
   )
 )
 
-# InteractiveCredential ----
+
 #' Interactive credential base class
 #'
 #' @description
@@ -232,12 +257,15 @@ InteractiveCredential <- R6::R6Class(
   classname = "InteractiveCredential",
   inherit = Credential,
   public = list(
+    #' @field interactive Logical indicating whether this credential requires
+    #'  user interaction.
+    interactive = TRUE,
     #' @description
-    #' Check if the credential is interactive
+    #' Check if the credential requires user interaction
     #'
-    #' @return Always returns `TRUE` for interactive credentials
+    #' @return Logical indicating whether this credential is interactive
     is_interactive = function() {
-      TRUE
+      self$interactive
     }
   )
 )
